"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs_1 = tslib_1.__importDefault(require("fs"));
var vscode_languageserver_1 = require("vscode-languageserver");
var vscode_uri_1 = require("vscode-uri");
var ignore_1 = tslib_1.__importDefault(require("ignore"));
var util_1 = require("../common/util");
var hunkStream_1 = tslib_1.__importDefault(require("../common/hunkStream"));
var path_1 = require("path");
var logger_1 = tslib_1.__importDefault(require("../common/logger"));
function handleFormat(config, textDocument, text, next) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var command, _a, rootPatterns, isStdout, isStderr, _b, args, ignoreExitCode, ignore, workDir, currentFile, relPath, cmd, _c, _d, stdout, _e, stderr, code, output;
        return tslib_1.__generator(this, function (_f) {
            switch (_f.label) {
                case 0:
                    command = config.command, _a = config.rootPatterns, rootPatterns = _a === void 0 ? [] : _a, isStdout = config.isStdout, isStderr = config.isStderr, _b = config.args, args = _b === void 0 ? [] : _b, ignoreExitCode = config.ignoreExitCode, ignore = config.ignore;
                    return [4 /*yield*/, util_1.findWorkDirectory(vscode_uri_1.URI.parse(textDocument.uri).fsPath, rootPatterns)];
                case 1:
                    workDir = _f.sent();
                    currentFile = vscode_uri_1.URI.parse(textDocument.uri).fsPath;
                    relPath = path_1.relative(workDir, currentFile);
                    try {
                        // ignore file
                        if (!path_1.isAbsolute(relPath) && ignore && ignore_1.default().add(ignore).ignores(relPath)) {
                            return [2 /*return*/, next(text)];
                        }
                    }
                    catch (err) {
                        logger_1.default.error("ignore error: " + (err.message || err.name || err));
                    }
                    if (config.requiredFiles && config.requiredFiles.length) {
                        if (!util_1.checkAnyFileExists(workDir, config.requiredFiles)) {
                            return [2 /*return*/, next(text)];
                        }
                    }
                    return [4 /*yield*/, util_1.findCommand(command, workDir)];
                case 2:
                    cmd = _f.sent();
                    return [4 /*yield*/, util_1.executeFile(new hunkStream_1.default(text), textDocument, cmd, args, {
                            cwd: workDir
                        })];
                case 3:
                    _c = _f.sent(), _d = _c.stdout, stdout = _d === void 0 ? '' : _d, _e = _c.stderr, stderr = _e === void 0 ? '' : _e, code = _c.code;
                    output = '';
                    if (!ignoreExitCode && code > 0) {
                        output = text;
                    }
                    else if (code > 0 && ignoreExitCode instanceof Array && ignoreExitCode.indexOf(code) === -1) {
                        output = text;
                    }
                    else if (config.doesWriteToFile) {
                        output = fs_1.default.readFileSync(vscode_uri_1.URI.parse(textDocument.uri).fsPath, 'utf8');
                    }
                    else if (isStdout === undefined && isStderr === undefined) {
                        output = stdout;
                    }
                    else {
                        if (isStdout) {
                            output += stdout;
                        }
                        if (isStderr) {
                            output += stderr;
                        }
                    }
                    return [2 /*return*/, next(output)];
            }
        });
    });
}
function formatDocument(formatterConfigs, textDocument, token) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var resolve, text;
        var _this = this;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    resolve = formatterConfigs
                        .reverse()
                        .reduce(function (res, config) {
                        return function (text) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                            var newText, err_1;
                            return tslib_1.__generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        if (token.isCancellationRequested) {
                                            return [2 /*return*/];
                                        }
                                        newText = text;
                                        _a.label = 1;
                                    case 1:
                                        _a.trys.push([1, 3, , 5]);
                                        return [4 /*yield*/, handleFormat(config, textDocument, text, res)];
                                    case 2:
                                        newText = _a.sent();
                                        return [3 /*break*/, 5];
                                    case 3:
                                        err_1 = _a.sent();
                                        logger_1.default.error("ignore error: " + (err_1.message || err_1.name || err_1));
                                        return [4 /*yield*/, res(text)];
                                    case 4:
                                        newText = _a.sent();
                                        return [3 /*break*/, 5];
                                    case 5: return [2 /*return*/, newText];
                                }
                            });
                        }); };
                    }, function (text) { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
                        return [2 /*return*/, text];
                    }); }); });
                    return [4 /*yield*/, resolve(textDocument.getText())];
                case 1:
                    text = _a.sent();
                    if (!text) {
                        return [2 /*return*/];
                    }
                    return [2 /*return*/, [{
                                range: vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(0, 0), vscode_languageserver_1.Position.create(textDocument.lineCount + 1, 0)),
                                newText: text
                            }]];
            }
        });
    });
}
exports.formatDocument = formatDocument;
